@@ -1,6 +1,6 @@
 /* exif-ifd.c
  *
- * Copyright  2002 Lutz Mller <lutz@users.sourceforge.net>
+ * Copyright (c) 2002 Lutz Mueller <lutz@users.sourceforge.net>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -18,12 +18,13 @@
  * Boston, MA 02111-1307, USA.
  */
 
-#include "config.h"
-#include "exif-ifd.h"
+#include <config.h>
+
+#include <libexif/exif-ifd.h>
 
 #include <stdlib.h>
 
-static struct {
+static const struct {
 	ExifIfd ifd;
 	const char *name;
 } ExifIfdTable[] = {
