@@ -1,6 +1,6 @@
 /* exif-log.c
  *
- * Copyright  2004 Lutz Mller <lutz@users.sourceforge.net>
+ * Copyright (c) 2004 Lutz Mueller <lutz@users.sourceforge.net>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -35,7 +35,7 @@ struct _ExifLog {
 	ExifMem *mem;
 };
 
-static struct {
+static const struct {
 	ExifLogCode code;
 	const char *title;
 	const char *message;
@@ -127,6 +127,10 @@ exif_log_set_func (ExifLog *log, ExifLogFunc func, void *data)
 	log->data = data;
 }
 
+#ifdef NO_VERBOSE_TAG_STRINGS
+/* exif_log forms part of the API and can't be commented away */
+#undef exif_log
+#endif
 void
 exif_log (ExifLog *log, ExifLogCode code, const char *domain,
 	  const char *format, ...)
